@@ -1,4 +1,4 @@
-#include <config.h>
+#include "config.h"
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -195,11 +195,12 @@ static gchar *types[] =
   "const ",
   "void",
   "gint",
-  "int ",
-  "char ",
+  " int ",
+  " char ",
   "gchar ",
   "gfloat",
   "float",
+  "double",
   "gint8",
   "gint16",
   "gint32",
@@ -234,6 +235,87 @@ static gchar *types[] =
   "GdkPixbuf ",
   "GError",
   "size_t",
+  "GtkAboutDialog ",
+  "GtkAction ",
+  "GtkActionEntry ",
+  "GtkRadioActionEntry ",
+  "GtkIconFactory ",
+  "GtkStockItem ",
+  "GtkIconSet ",
+  "GtkTextBuffer ",
+  "GtkStatusbar ",
+  "GtkTextIter ",
+  "GtkTextMark ",
+  "GdkEventWindowState ",
+  "GtkActionGroup ",
+  "GtkUIManager ",
+  "GtkRadioAction ",
+  "GtkActionClass ",
+  "GtkToggleActionEntry ",
+  "GtkAssistant ",
+  "GtkBuilder ",
+  "GtkSizeGroup ",
+  "GtkTreeModel ",
+  "GtkTreeSelection ",
+  "GdkDisplay ",
+  "GdkScreen ",
+  "GdkWindow ",
+  "GdkEventButton ",
+  "GdkCursor ",
+  "GtkTreeIter ",
+  "GtkTreeViewColumn ",
+  "GdkDisplayManager ",
+  "GtkClipboard ",
+  "GtkIconSize ",
+  "GtkImage ",
+  "GdkDragContext ",
+  "GtkSelectionData ",
+  "GtkDialog ",
+  "GtkMenuItem ",
+  "GtkListStore ",
+  "GtkCellLayout ",
+  "GtkCellRenderer ",
+  "GtkTreePath ",
+  "GtkTreeStore ",
+  "GtkEntry ",
+  "GtkEditable ",
+  "GtkEditableClass ",
+  "GdkPixmap ",
+  "GdkEventConfigure ",
+  "GdkEventMotion ",
+  "GdkModifierType ",
+  "GtkEntryCompletion ",
+  "GtkToolItem ",
+  "GDir ",
+  "GtkIconView ",
+  "GtkCellRendererText ",
+  "GtkContainer ",
+  "GtkAccelGroup ",
+  "GtkPaned ",
+  "GtkPrintOperation ",
+  "GtkPrintContext ",
+  "cairo_t ",
+  "PangoLayout "
+  "PangoFontDescription ",
+  "PangoRenderer ",
+  "PangoMatrix ",
+  "PangoContext ",
+  "PangoLayout ",
+  "GtkTable ",
+  "GtkToggleButton ",
+  "GString ",
+  "GtkIconSize ",
+  "GtkTreeView ",
+  "GtkTextTag ",
+  "GdkEvent ",
+  "GdkEventKey ",
+  "GtkTextView ",
+  "GdkEventVisibility ",
+  "GdkBitmap ",
+  "GtkTextChildAnchor ",
+  "GArray ",
+  "GtkCellEditable ",
+  "GtkCellRendererToggle ",
   NULL
 };
 
@@ -308,7 +390,8 @@ parse_chars (gchar     *text,
     }
   /* check for types */
   for (i = 0; types[i] != NULL; i++)
-    if (!strncmp (text, types[i], strlen (types[i])))
+    if (!strncmp (text, types[i], strlen (types[i])) ||
+        (start && types[i][0] == ' ' && !strncmp (text, types[i] + 1, strlen (types[i]) - 1)))
       {
 	*end_ptr = text + strlen (types[i]);
 	*tag = "type";
@@ -575,6 +658,8 @@ load_file (const gchar *filename)
 	}
     }
 
+  fclose (file);
+  
   fontify ();
 
   g_string_free (buffer, TRUE);
