@@ -22,7 +22,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -62,6 +62,10 @@
 #include <X11/extensions/Xdamage.h>
 #endif
 
+#ifdef HAVE_RANDR
+#include <X11/extensions/Xrandr.h>
+#endif
+
 
 static void   gdk_display_x11_dispose            (GObject            *object);
 static void   gdk_display_x11_finalize           (GObject            *object);
@@ -82,9 +86,16 @@ static const char *const precache_atoms[] = {
   "UTF8_STRING",
   "WM_CLIENT_LEADER",
   "WM_DELETE_WINDOW",
+  "WM_ICON_NAME",
   "WM_LOCALE_NAME",
+  "WM_NAME",
   "WM_PROTOCOLS",
   "WM_TAKE_FOCUS",
+  "WM_WINDOW_ROLE",
+  "_NET_ACTIVE_WINDOW",
+  "_NET_CURRENT_DESKTOP",
+  "_NET_FRAME_EXTENTS",
+  "_NET_STARTUP_ID",
   "_NET_WM_CM_S0",
   "_NET_WM_DESKTOP",
   "_NET_WM_ICON",
@@ -93,10 +104,15 @@ static const char *const precache_atoms[] = {
   "_NET_WM_PID",
   "_NET_WM_PING",
   "_NET_WM_STATE",
-  "_NET_WM_STATE_STICKY",
+  "_NET_WM_STATE_ABOVE",
+  "_NET_WM_STATE_BELOW",
+  "_NET_WM_STATE_FULLSCREEN",
+  "_NET_WM_STATE_MODAL",
   "_NET_WM_STATE_MAXIMIZED_VERT",
   "_NET_WM_STATE_MAXIMIZED_HORZ",
-  "_NET_WM_STATE_FULLSCREEN",
+  "_NET_WM_STATE_SKIP_TASKBAR",
+  "_NET_WM_STATE_SKIP_PAGER",
+  "_NET_WM_STATE_STICKY",
   "_NET_WM_SYNC_REQUEST",
   "_NET_WM_SYNC_REQUEST_COUNTER",
   "_NET_WM_WINDOW_TYPE",
@@ -165,6 +181,23 @@ gdk_display_open (const gchar *display_name)
   XAddConnectionWatch (xdisplay, gdk_internal_connection_watch, NULL);
 #endif /* HAVE_X11R6 */
   
+  _gdk_x11_precache_atoms (display, precache_atoms, G_N_ELEMENTS (precache_atoms));
+
+  /* RandR must be initialized before we initialize the screens */
+  display_x11->have_randr12 = FALSE;
+#ifdef HAVE_RANDR
+  if (XRRQueryExtension (display_x11->xdisplay,
+			 &display_x11->xrandr_event_base, &ignore))
+  {
+      int major, minor;
+      
+      XRRQueryVersion (display_x11->xdisplay, &major, &minor);
+
+      if ((major == 1 && minor >= 2) || major > 1)
+	  display_x11->have_randr12 = TRUE;
+  }
+#endif
+  
   /* initialize the display's screens */ 
   display_x11->screens = g_new (GdkScreen *, ScreenCount (display_x11->xdisplay));
   for (i = 0; i < ScreenCount (display_x11->xdisplay); i++)
@@ -187,8 +220,6 @@ gdk_display_open (const gchar *display_name)
   attr.height = 10;
   attr.event_mask = 0;
 
-  _gdk_x11_precache_atoms (display, precache_atoms, G_N_ELEMENTS (precache_atoms));
-
   display_x11->leader_gdk_window = gdk_window_new (GDK_SCREEN_X11 (display_x11->default_screen)->root_window, 
 						   &attr, GDK_WA_X | GDK_WA_Y);
   (_gdk_x11_window_get_toplevel (display_x11->leader_gdk_window))->is_leader = TRUE;
@@ -217,7 +248,17 @@ gdk_display_open (const gchar *display_name)
 #ifdef HAVE_XCOMPOSITE
   if (XCompositeQueryExtension (display_x11->xdisplay,
 				&ignore, &ignore))
-      display_x11->have_xcomposite = TRUE;
+    {
+      int major, minor;
+              
+      XCompositeQueryVersion (display_x11->xdisplay, &major, &minor);
+
+      /* Prior to Composite version 0.4, composited windows clipped their
+       * parents, so you had to use IncludeInferiors to draw to the parent
+       * This isn't useful for our purposes, so require 0.4
+       */
+      display_x11->have_xcomposite = major > 0 || (major == 0 && minor >= 4);
+    }
   else
 #endif
     display_x11->have_xcomposite = FALSE;
@@ -896,6 +937,7 @@ _gdk_x11_display_screen_for_xrootwin (GdkDisplay *display,
 Display *
 gdk_x11_display_get_xdisplay (GdkDisplay *display)
 {
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
   return GDK_DISPLAY_X11 (display)->xdisplay;
 }
 
@@ -1280,11 +1322,11 @@ gdk_display_supports_clipboard_persistence (GdkDisplay *display)
  * Since: 2.6
  */
 void
-gdk_display_store_clipboard (GdkDisplay *display,
-			     GdkWindow  *clipboard_window,
-			     guint32     time_,
-			     GdkAtom    *targets,
-			     gint        n_targets)
+gdk_display_store_clipboard (GdkDisplay    *display,
+			     GdkWindow     *clipboard_window,
+			     guint32        time_,
+			     const GdkAtom *targets,
+			     gint           n_targets)
 {
   GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (display);
   Atom clipboard_manager, save_targets;
